/* MasterInspector.m				 
 *
 * This is the BeakerBoys Master Inspector. Here all the other Inspectors
 * are created or have to register themselves.
 *
 * For interface-info see the header file. The comments in this file mostly
 * cover only the real implementation details.
 *
 * Written by: 		Thomas Engel
 * Created:    		20.11.1993 (Copyleft)
 * Last modified: 	08.02.1994
 */

#import "MasterInspector.h"

#import "BasicAtomInspector.subproj/BasicAtomInspector.h"
#import "RegionInspector.subproj/RegionInspector.h"
#import "MoleculeInspector.subproj/MoleculeInspector.h"
#import "AtomInspector.subproj/AtomInspector.h"
#import "BondInspector.subproj/BondInspector.h"
#import "BeakerInspector.subproj/BeakerInspector.h"
#import "CameraInspector.subproj/CameraInspector.h"

@implementation MasterInspector

- addDefaultInspectors
{
	// Let's add our inspectors after our super did.
	// Always remember. The order is important! Adding it later gives it
	// a higher priorty.
	
	[super addDefaultInspectors];
	
	[[BasicAtomInspector new] setManager:self];
	[[RegionInspector new] setManager:self];
	[[MoleculeInspector new] setManager:self];
	[[AtomInspector new] setManager:self];
	[[BondInspector new] setManager:self];
	[[BeakerInspector new] setManager:self];
	[[CameraInspector new] setManager:self];
	
	return self;
}


@end

/*
 * History: 08.02.94 Complete redesign to work with MiscInspectorManager. A
 *					 lot easyier now.
 *
 *			08.01.94 Changed to work with our new swapControllers.
 *
 *			20.12.93 Copied the code from the old InspectorManager and added
 *					 the default inspectors.
 *
 *
 * Bugs: Not here
 */