/**************************************************************************
 * CLASS:		MiscIconWell
 * PROGRAMMER:	Todd Thomas		(todd@avocado.cuc.ab.ca)
 * DATE:		May 24, 1994
 * VERSION:		0.4
 *
 *	This class (by default) allows both source and destination dragging of 
 * icon representations of files and folders and uses the NXFilenamePboardType
 * to pass the filename that is associated with the dragged icon.
 * Since we are inheriting the basic functionality from MiscDragView, we 
 * should not need to do much. 
 *
 * Small bug fixes: BJM 5/24/94 (Bruce McKenzie, spuds@netcom.com)
 *   1) changed "- (char *)filename" to "- (const char *)filename"
 *   2) fixed recognition of multiple files in filename to look for tab,
 *       rather than "any whitespace".
 *   3) fixed places where it was assumed that filename ivar was ever nil
 *   4) Added proper archiving/versioning to class.
 *   5) Added "- launch:sender" method
 *
 * This object is included in the MiscKit by permission from the author
 * and its use is governed by the MiscKit license, found in the file
 * "LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
 * for a list of all applicable permissions and restrictions.
 ***************************************************************************/
 
#import <appkit/appkit.h>
#import <misckit/MiscDragView.h>


@interface MiscIconWell : MiscDragView
{
	id  filename;
	BOOL  allowDoubleClickLaunch;
}

- initFrame: (const NXRect *)frameRect;
- awake;
- free;
- (const char *)filename;
- setImageByFilename: (const char *)aFilename;
- setAllowDoubleClickLaunch: (BOOL)aBool;
- (BOOL)allowDoubleClickLaunch;

/*
 * Launch/open file in WS: BJM 5/24/94
 */
- launch:sender;

- mouseDown: (NXEvent *)theEvent;
- (BOOL)setupForSourceDrag;
- (BOOL)performDragOperation: sender;
- concludeDragOperation: sender;
- read: (NXTypedStream *)stream;
- write: (NXTypedStream *)stream;

@end
