//
//	MiscSliderCell.h -- a TextFieldCell subclass that adds a SliderCell.
//	   Written by David Fedchenko. Copyright 1994 by David Fedchenko.
//	       		 Version 1.0  All rights reserved.
//
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <appkit/appkit.h>

#define MSC_SPACING      4
#define MSC_WIDTH        16
#define MSC_MINWIDTH     30
// these values are the cell tags from the matrix in the inspector
#define MSC_ABOVELEFT    1
#define MSC_ABOVECENTER  2
#define MSC_ABOVERIGHT   3
#define MSC_LEFT         4
#define MSC_RIGHT        5
#define MSC_BELOWLEFT    6
#define MSC_BELOWCENTER  7
#define MSC_BELOWRIGHT   8

@interface MiscSliderCell:TextFieldCell
    {
    id      (idSlider);
    id      stringList;
    
    int     layout;
    float   split;
    double  actualValue;
    double  limitMinValue;
    double  limitMaxValue;
    double  limitMinBound;
    double  limitMaxBound;
    BOOL    fExpandMin;
    BOOL    fExpandMax;
    BOOL    fInt;
    
    BOOL    fLoop;
    BOOL    fFeedback;
    BOOL    fValidRect;
    NXRect  rectFrame;
    }

- initTextCell:(const char *)sz;
- copyFromZone:(NXZone *)zone;

- drawInside:(const NXRect *)aRect inView:controlView;
- drawSelf:(const NXRect *)cellFrame inView:controlView;
- resetCursorRect:(const NXRect *)cellFrame inView:aView;

- calcCellSize:(NXSize *)theSize inRect:(const NXRect *)aRect;
- edit:(const NXRect *)aRect inView:controlView editor:textObj
    delegate:anObject event:(NXEvent *)theEvent;
- getDrawRect:(NXRect *)theRect;
- getTitleRect:(NXRect *)theRect;
- highlight:(const NXRect *)cellFrame inView:aView lit:(BOOL)flag;
- select:(const NXRect *)aRect inView:aView editor:aTextObject
    delegate:anObject start:(int)selStart length:(int)selLength;
- setEditable:(BOOL)flag;

- setIntValue:(int)anInt;
- setFloatValue:(float)aFloat;
- setDoubleValue:(double)anDouble;
- setStringValue:(const char *)aString;
-(int) intValue;
-(float) floatValue;
-(double) doubleValue;

- moved:sender;

- setMinValue:(double)value;
- setMaxValue:(double)value;
- setMinBoundary:(double)value;
- setMaxBoundary:(double)value;
-(double) minValue;
-(double) maxValue;
-(double) minBoundary;
-(double) maxBoundary;
- setExpandMin:(BOOL)flag;
- setExpandMax:(BOOL)flag;
-(BOOL) expandMin;
-(BOOL) expandMax;
- setPosition:(int)where;
- setSplit:(int)percent;
- setIntegerOnly:(BOOL)flag;
-(int) position;
-(int) split;
-(BOOL) integerOnly;

- setStringList:anObject;
- stringList;

- write:(NXTypedStream *)stream;
- read:(NXTypedStream *)stream;
- awake;
- awakeFromNib;

@end

@interface Object(MiscValueDelegate)
-(const char *) stringAt:(int)item;
@end
