//----------------------------------------------------------------------------------------------------
//
//	CalendarClock
//
//	Inherits From:		Clock
//
//	Declared In:		CalendarClock.h
//
//	Class Description
//
//		CalendarClock adds behavior required for specific graphical
//		representation.  A CalendarClock is a fixed size view (64 x 64).
//		It uses a specific clock face image and draws time and date 
//		information based on that image's appearance.
//
//
//	Disclaimer
//
//		You may freely copy, distribute and reuse this software and its
//		associated documentation. I disclaim any warranty of any kind, 
//		expressed or implied, as to its fitness for any particular use.
//
//----------------------------------------------------------------------------------------------------
#import "Clock.h"


@interface CalendarClock : Clock
{
	NXColor		timeColor;
	NXColor		weekdayColor;
	NXColor		dateColor;
	NXColor		monthColor;
	BOOL		wantsYear;
}


//----------------------------------------------------------------------------------------------------
//	Accessing Display Colors
//----------------------------------------------------------------------------------------------------
- timeColor: (NXColor) aColor;
- weekdayColor: (NXColor) aColor;
- dateColor: (NXColor) aColor;
- monthColor: (NXColor) aColor;

- (NXColor) timeColor;
- (NXColor) weekdayColor;
- (NXColor) dateColor;
- (NXColor) monthColor;


//----------------------------------------------------------------------------------------------------
//	Enabling Year Display
//-----------------------------------------------------------------------------------------------------
- wantsYear: (BOOL) aFlag;
- (BOOL) wantsYear;


//----------------------------------------------------------------------------------------------------
//	Action Methods
//----------------------------------------------------------------------------------------------------
- takeWantsYearFlagFrom :sender;

- takeTimeColorFrom: sender;
- takeWeekdayColorFrom: sender;
- takeDateColorFrom: sender;
- takeMonthColorFrom: sender;


@end