//---------------------------------------------------------------------------------------------------------
//
//	KeyDownBrowser
//
//	Inherits From:		NXBrowser
//
//	Declared In:		KeyDownBrowser.h
//
//	Class Description
//
//		KeyDownBrowser is a subclass of NXBrowser extended to respond
//		to keyboard entry.  Arrow keys may be used for manuevering.  Other
//		keystrokes are interpreted as search criteria.  The browser will attempt
//		to find a match for these keystrokes within the appropriate column.
//		The search will not begin until you are finished entering keystrokes. 
//		The browser uses the inputTimeOut instance variable to 'know' when 
//		this is (i.e. the search will begin inputTimeOut seconds after the last 
//		keystroke. 0.5 seconds is the default.). Selection within the browser 
//		by arrow keys or by keystroke matching is equivalent to a single mouse 
//		click and sends the action to the target.  Pressing the Return key is 
//		equivalent to a double-click and sends the double action to the target.  
//		KeyDownBrowser overrides NXBrowser's default arrow key behavior, 
//		instead providing the more intuitive behavior found in the Workspace 
//		Manager browser.  Rules for search behavior were modeled after WM 
//		as well.
//		
//
//	Disclaimer
//
//		You may freely copy, distribute and reuse this software and its
//		associated documentation. I disclaim any warranty of any kind, 
//		expressed or implied, as to its fitness for any particular use.
//
//---------------------------------------------------------------------------------------------------------
#import <appkit/appkit.h>

@interface KeyDownBrowser : NXBrowser

{
	STR				searchString;
	BOOL			wantsCaseSensitiveSearch;
	float				inputTimeOut;
	DPSTimedEntry	timedEntry;
}


//---------------------------------------------------------------------------------------------------------
//	 Accessors
//---------------------------------------------------------------------------------------------------------
- wantsCaseSensitiveSearch: (BOOL) aFlag;
- (BOOL) wantsCaseSensitiveSearch;

- inputTimeOut: (float) aFloat;
- (float) inputTimeOut;


@end